----------------------------------------------------------------------------
--                                  M2OS
--
--                           Copyright (C) 2019
--                    Universidad de Cantabria, SPAIN
--
--  Author: Mario Aldea Rivas (aldeam@unican.es)
--
--  This file is part of M2OS.
--
--  M2OS is free software: you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation, either version 3 of the License, or
--  (at your option) any later version.
--
--  M2OS is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with M2OS.  If not, see <https://www.gnu.org/licenses/>.
----------------------------------------------------------------------------
with M2.Kernel.Ready_Queue;
with M2.HAL;
with M2.Kernel.TCBs;
with M2.Debug;


package body M2.Kernel.Dispatcher is
   package RQ renames Ready_Queue;
   use type TCBs.TCB_Ac;

   ------------------
   -- Show_Running --
   ------------------

   procedure Show_Running;

   procedure Show_Running is
   begin
      if RQ.Running_Ac /= TCBs.Null_TCB_Ac then
         pragma Debug (M2.Debug.Put ("RT"));
         TCBs.Show (RQ.Running_Ac.all);
      else
         pragma Debug (M2.Debug.Put ("RT(null)"));
      end if;
   end Show_Running;

   -------------
   -- Show_CS --
   -------------

   procedure Show_CS (Heir_Ac : TCBs.TCB_Ac);

   procedure Show_CS (Heir_Ac : TCBs.TCB_Ac) is
   begin
      pragma Debug (M2.Debug.Put ("CS->", Integer (Heir_Ac.Id)));
   end Show_CS;

   --------------
   -- Dispatch --
   --------------

   procedure Dispatch is
      Head_Ac : TCBs.TCB_Ac;

   begin
      pragma Debug (M2.Debug.Put ("|Disp"));
      pragma Debug (Show_Running);
      pragma Debug (RQ.Show (M2.Trace));
      pragma Assert (RQ.Running_Ac = TCBs.Null_TCB_Ac or else
                     RQ.Running_Ac.Status = Ready);
      pragma Assert (HAL.Get_Current_Stack_Margin_In_Bytes > 50);
      pragma Assert (HAL.Get_Current_Stack_Size_In_Bytes <
                       HAL.Get_Max_Stack_Usage_In_Bytes);

      --  Wait until there is some active task.

      loop
         RQ.Activate_Suspended_Threads;

         Head_Ac := RQ.Head_Of_Ready_Queue;

         if Head_Ac /= TCBs.Null_TCB_Ac then
            pragma Assert (Head_Ac.Status = Ready);

            if RQ.Running_Ac = TCBs.Null_TCB_Ac then
               --  Change ready thread
               RQ.Running_Ac := Head_Ac;
               RQ.Dequeue_Head_Of_Ready_Queue;

            elsif Head_Ac.Priority > RQ.Running_Ac.Priority then

               RQ.Dequeue_Head_Of_Ready_Queue;
               RQ.Enqueue_Ready_Thread (RQ.Running_Ac);
               RQ.Running_Ac := Head_Ac;
            end if;
         end if;

         exit when RQ.Running_Ac /= TCBs.Null_TCB_Ac;
      end loop;

      --  Perform context switch to the Heir task

      pragma Debug (Show_CS (RQ.Running_Ac));
      HAL.Context_Switch (New_Context => RQ.Running_Ac.Context);
   end Dispatch;

end M2.Kernel.Dispatcher;
