----------------------------------------------------------------------------
--                                  M2OS
--
--                           Copyright (C) 2019
--                    Universidad de Cantabria, SPAIN
--
--  Author: Mario Aldea Rivas (aldeam@unican.es)
--
--  This file is part of M2OS.
--
--  M2OS is free software: you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation, either version 3 of the License, or
--  (at your option) any later version.
--
--  M2OS is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with M2OS.  If not, see <https://www.gnu.org/licenses/>.
----------------------------------------------------------------------------
with M2.HAL;
with System;  --  M2OS to solve a GNAT internal error
--  with M2.Kernel.Timing_Events;

package M2.Kernel.TCBs is

   --  pragma No_Elaboration_Code_All;
   --  pragma Preelaborate;

   type TCB; -- is limited private;
   --  Thread Control Block

   type TCB_Ac is access all TCB;

   Null_TCB_Ac : constant TCB_Ac;

   procedure Initialize_TCB
     (Th_Ac : TCB_Ac;
      Id : Kernel.Thread_Id;
      Priority : M2.Kernel.Priority;
      Init_Body_Ac : Kernel.Thread_Init_Body_Ac; --HAL.Procedure_Ac;
      Args : System.Address);

   function Id (Th : in TCB)
                return Kernel.Thread_Id;
   --  with Inline_Always;

   procedure Set_Status (Th : in out TCB;
                         St : Thread_Status)
     with Inline_Always;

   procedure Set_Job_Body (Th : in out TCB;
                           Job_Body_Ac : HAL.Procedure_Ac)
     with Inline_Always;

   function Status (Th : in TCB)
                    return Thread_Status
     with Inline_Always;

   function Context (Th : in TCB)
                     return HAL.Thread_Context;

   function Priority (Th : in TCB)
                      return Kernel.Priority
     with Inline_Always;

   procedure Show  (Th : in TCB; Level : M2.Trace_Levels := M2.Trace);

--private

   type TCB is record
      Next : TCB_Ac;

      Context : aliased HAL.Thread_Context; -- := (others => null);

      Priority : Kernel.Priority; -- := 0;

      Id : Kernel.Thread_Id; -- := Kernel.Invalid_Thread_Id;

      Status : M2.Kernel.Thread_Status; -- := Blocked;

      Init_Body_Ac : Kernel.Thread_Init_Body_Ac;
      --  Initial body of the thread: it is the main body of the Ada task that
      --  receives an Address parameter with the task type discriminants (in
      --  the case the task has been created from a task type with
      --  discriminants).
      --  Only executed in the first activation when called from
      --  Scheduler.Thread_Body_Wrapper, subsecuent activations execute the
      --  "job body" stored in context by API.Set_Job_Body_Of_Running_Thread.

      Args : System.Address;

      Activation_Time : HAL.HWTime;
   end record;
   pragma Suppress_Initialization (TCB);

private
   Null_TCB_Ac : constant TCB_Ac := null;

end M2.Kernel.TCBs;
