----------------------------------------------------------------------------
--                                  M2OS
--
--                           Copyright (C) 2019
--                    Universidad de Cantabria, SPAIN
--
--  Author: Mario Aldea Rivas (aldeam@unican.es)
--
--  This file is part of M2OS.
--
--  M2OS is free software: you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation, either version 3 of the License, or
--  (at your option) any later version.
--
--  M2OS is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with M2OS.  If not, see <https://www.gnu.org/licenses/>.
----------------------------------------------------------------------------
with Ada.Unchecked_Conversion;

package body M2.Queue is
   package QImpl renames M2.Queue_Implementation;
   type Element_Ac is access all Element;
   type Queue_Ac is access all Queue;

   type Mayor_Than_Function is
     access function (Left, Right : access Element) return Boolean;

   function To_E Is new Ada.Unchecked_Conversion
     (Source => QImpl.Base_Element_Ac,
      Target => Element_Ac);

   pragma Warnings (Off);
   function To_B is new Ada.Unchecked_Conversion
     (Source => Element_Ac,
      Target => QImpl.Base_Element_Ac);

   function To_BQ is new Ada.Unchecked_Conversion
     (Source => Queue_Ac,
      Target => QImpl.Queue_Ac);
   pragma Warnings (On);

   function To_MT is new Ada.Unchecked_Conversion
     (Source => Mayor_Than_Function,
      Target => QImpl.Mayor_Than_Function_Ac);

   -------------
   -- Enqueue --
   -------------

   procedure Enqueue (E : access Element;
                      Q : access Queue) is
   begin
      QImpl.Enqueue (To_B (E), To_BQ (Q), To_MT (Mayor_Than));
   end Enqueue;

   -------------
   -- Dequeue --
   -------------

   procedure Dequeue (E : access Element;
                      Q : access Queue) is
   begin
      QImpl.Dequeue (To_B (E), To_BQ (Q));
   end Dequeue;

   ------------------
   -- Dequeue_Head --
   ------------------

   procedure Dequeue_Head (Q : access Queue) is
   begin
      QImpl.Dequeue_Head (To_BQ (Q));
   end Dequeue_Head;

   ----------
   -- Head --
   ----------

   function Head (Q : access Queue) return access Element is
   begin
      return To_E (To_BQ (Q).all);
   end Head;

end M2.Queue;
