----------------------------------------------------------------------------
--                                  M2OS
--
--                           Copyright (C) 2019
--                    Universidad de Cantabria, SPAIN
--
--  Author: Mario Aldea Rivas (aldeam@unican.es)
--
--  This file is part of M2OS.
--
--  M2OS is free software: you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation, either version 3 of the License, or
--  (at your option) any later version.
--
--  M2OS is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with M2OS.  If not, see <https://www.gnu.org/licenses/>.
----------------------------------------------------------------------------
package body M2.Queue_Implementation is

   -------------
   -- Enqueue --
   -------------

   procedure Enqueue (E : Base_Element_Ac;
                      Q : Queue_Ac;
                      Mayor_Than : Mayor_Than_Function_Ac) is
   begin
      pragma Assert (Q /= null);

      if Q.all = Null or else Mayor_Than (Q.all, E) then
         E.Next := Q.all;
         Q.all := E;

      else
         declare
            Aux : Base_Element_Ac := Q.all;
         begin
            loop
               pragma Assert (Aux = null or else Aux.Next = null or else
                              not Mayor_Than (Aux, Aux.Next));

               if Aux.Next = Null or else Mayor_Than (Aux.Next, E) then
                  E.Next := Aux.Next;
                  Aux.Next := E;
                  exit;
               end if;
               Aux := Aux.Next;
            end loop;
         end;
      end if;
   end Enqueue;

   -------------
   -- Dequeue --
   -------------

   procedure Dequeue (E : Base_Element_Ac;
                      Q : Queue_Ac) is
   begin
      pragma Assert (Q /= null);
      pragma Assert (Q.all /= null);

      if E = Q.all then
         Q.all := Q.all.Next;

      else
         declare
            Aux : Base_Element_Ac := Q.all;
         begin
            loop
               pragma Assert (Aux /= null);

               if Aux.Next = E then
                  Aux.Next := E.Next;
                  exit;
               end if;
               Aux := Aux.Next;
            end loop;
         end;
      end if;

   end Dequeue;

   ------------------
   -- Dequeue_Head --
   ------------------

   procedure Dequeue_Head (Q : Queue_Ac) is
   begin
      pragma Assert (Q /= null);
      pragma Assert (Q.all /= null);

      Q.all := Q.all.Next;
   end Dequeue_Head;

end M2.Queue_Implementation;
