----------------------------------------------------------------------------
--                                  M2OS
--
--                           Copyright (C) 2019
--                    Universidad de Cantabria, SPAIN
--
--  Author: Mario Aldea Rivas (aldeam@unican.es)
--
--  This file is part of M2OS.
--
--  M2OS is free software: you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation, either version 3 of the License, or
--  (at your option) any later version.
--
--  M2OS is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with M2OS.  If not, see <https://www.gnu.org/licenses/>.
----------------------------------------------------------------------------

pragma Ravenscar;
pragma Task_Dispatching_Policy (FIFO_Within_Priorities);
pragma Locking_Policy(Ceiling_Locking);
pragma Queuing_Policy(Priority_Queuing);

with System;

with Ada.Real_Time;

with M2.Direct_IO;
with M2.HAL;
with Tests_Reports;

with Timing_Events_And_Tasks_PO;
with Timing_Events_And_Tasks_Tasks;

procedure Timing_Events_And_Tasks_Test  is
   package DIO renames M2.Direct_IO;
   package RT renames Ada.Real_Time;

   use type Ada.Real_Time.Time;

   Handler1_Period : constant := 0.5;
   Handler2_Period : constant := 0.25;

   T : RT.Time := RT.Clock;

begin
   DIO.Put_Line ("-Timing_Events_And_Tasks_Test-");

   DIO.Put_Line ("Program TEs");

   Timing_Events_And_Tasks_PO.PO.Start1 (T + RT.To_Time_Span (0.25),
                                         RT.To_Time_Span (Handler1_Period));

   Timing_Events_And_Tasks_PO.PO.Start2 (T + RT.To_Time_Span (0.5),
                                         RT.To_Time_Span (Handler2_Period));

   Tests_Reports.Assert (Timing_Events_And_Tasks_PO.Step = 0);
   Timing_Events_And_Tasks_PO.Step := Timing_Events_And_Tasks_PO.Step + 1;

   DIO.Put_Line ("Wait for tasks and TEs");

end Timing_Events_And_Tasks_Test;
