----------------------------------------------------------------------------
--                                  M2OS
--
--                           Copyright (C) 2019
--                    Universidad de Cantabria, SPAIN
--
--  Author: Mario Aldea Rivas (aldeam@unican.es)
--
--  This file is part of M2OS.
--
--  M2OS is free software: you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation, either version 3 of the License, or
--  (at your option) any later version.
--
--  M2OS is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with M2OS.  If not, see <https://www.gnu.org/licenses/>.
----------------------------------------------------------------------------

with Interfaces;

with Tests_Reports;
with GNAT.Source_Info;

package Measurements is
   
#if M2_TARGET = "arduino_uno" then
   type Measurement_Types is (Long, Short);
   --  Long: uses the milliseconds count clock (1ms resolution)
   --  Short: uses the timer counter (4us resolution, 1ms max)
#elsif M2_TARGET = "stm32f4" or M2_TARGET = "epiphany" or M2_TARGET = "avr_iot" then
   type Measurement_Types is (Long);
#else
   pragma Compile_Time_Error (True, "Unexpected architecture");
#end if;
   
   type Time is new Interfaces.Unsigned_32;
   --  The value of a unit of this Time type depends on the target and on
   --  the Measurement_Type.
   --  Can be obtained with function Time_Unit_in_US.

   procedure Init (Measurement_Type : Measurement_Types);
   
   procedure Reset;
   
   procedure Start_Measurement
     with Inline_Always;
   
   procedure End_Measurement
     with Inline_Always;
   
   function Measurement_Time return Time;
   
   function Time_Unit_In_Us return Time;
   --  Length in microseconds of a time unit
   
   function Measurement_Avg_Time_Us return Time; -- in microseconds
   
   procedure Assert
     (Condition : Boolean;
      Msg       : String := GNAT.Source_Info.Source_Location)
     renames Tests_Reports.Assert;
   
   procedure Assert_Equal
     (Val1 : Integer; Val2 : Integer; Margin : Integer;
      Msg       : String := GNAT.Source_Info.Source_Location)
     renames Tests_Reports.Assert_Equal;
   
   procedure Finish;

end Measurements;
